#version 330 core
/*
    Voxelverse

    Copyright (c) 2022, Voxelverse Team.
    All rights reserved.
        
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        
    Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
    
    Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

uniform sampler2D uNormalSampler;
uniform sampler2D uPosSampler;
uniform sampler2D uSpecularSampler;
uniform mat4 uModelMatrix;

uniform vec3 uLightColor;
uniform vec3 uLightSource;

in vec2 passTexCoords;

layout (location = 0) out vec4 outColor;

void main()
{
    const float shininess = 256.0;
    const float attenuation = 0.8;

    vec3 lightSource = (uModelMatrix * vec4(uLightSource, 1.0)).xyz;

    vec3 normal = texture(uNormalSampler, passTexCoords).xyz;
    vec3 pos = texture(uPosSampler, passTexCoords).xyz;
    float specular = texture(uSpecularSampler, passTexCoords).x;

    vec3 viewDir = normalize(-pos);
    vec3 reflectDir = reflect(normalize(pos - lightSource), normal);

    vec3 lightRay = lightSource - pos;
    float directedIntensity = max(0.0, dot(normalize(lightRay), normal)) + 0.7;
    vec3 color = uLightColor * clamp(directedIntensity / (attenuation * dot(lightRay, lightRay)), 0.0, 1.0)
        + vec3(specular * pow(max(dot(viewDir, reflectDir), 0.0), shininess) / (attenuation * length(lightRay)));

    outColor = vec4(color, 1.0);
}