#version 330 core
/*
    Voxelverse

    Copyright (c) 2022, Voxelverse Team.
    All rights reserved.
        
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        
    Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
    
    Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

uniform sampler2D uColorSampler;
uniform sampler2D uNormalSampler;
uniform sampler2D uLightSampler;
uniform sampler2D uSpecularSampler;

in vec2 passTexCoords;
in vec3 passNormal;
in vec4 passPos;
in vec3 passUVec;
in vec3 passVVec;

layout (location = 0) out vec4 outColor;
layout (location = 1) out vec4 outNormal;
layout (location = 2) out vec4 outPos;
layout (location = 3) out vec4 outLight;
layout (location = 4) out vec4 outSpecular;

void main()
{
    vec4 color = texture(uColorSampler, passTexCoords);
    if (color.a < 0.1)
    {
        discard;
    }

    vec3 rawNormal = normalize(2.0 * texture(uNormalSampler, passTexCoords).xyz - vec3(1.0));
    vec3 normal = rawNormal.x * passUVec + rawNormal.y * passVVec + rawNormal.z * passNormal;

    outColor = color;
    outNormal = vec4(normal, 1.0);
    outPos = passPos;
    outLight = vec4(texture(uLightSampler, passTexCoords).xyz, 1.0);
    outSpecular = vec4(texture(uSpecularSampler, passTexCoords).xyz, 1.0);
}